@@ -21,8 +21,11 @@
 
 
 #include "metadatamusicbackend.h"
+#include "alwaysexpandedmetadatamodel.h"
+#include "../metadatabackendcommonmodel.h"
 
 #include <libs/mediacenter/pmcmetadatamodel.h>
+#include <libs/mediacenter/playlistmodel.h>
 
 #include <Nepomuk2/Vocabulary/NMM>
 #include <Nepomuk2/Vocabulary/NFO>
@@ -30,10 +33,20 @@
 #include <Nepomuk2/Query/ComparisonTerm>
 #include <Nepomuk2/Query/ResourceTypeTerm>
 #include <Nepomuk2/Query/ResourceTerm>
-#include <mediacenter/playlistmodel.h>
+
+#include <KDebug>
+
+#include <QDeclarativeEngine>
+#include <QDeclarativeContext>
+#include <QTimer>
 
 MEDIACENTER_EXPORT_BROWSINGBACKEND(MetadataMusicBackend)
 
+namespace {
+    static const char *s_showAllButton = "Show All";
+    static const char *s_playAllButton = "Play All";
+}
+
 MetadataMusicBackend::MetadataMusicBackend(QObject* parent, const QVariantList& args)
     : AbstractMetadataBackend(parent, args)
     , m_artistsModel(0)
@@ -51,24 +64,39 @@ MetadataMusicBackend::~MetadataMusicBackend()
 bool MetadataMusicBackend::initImpl()
 {
     AbstractMetadataBackend::initImpl();
-    m_albumsModel = new PmcMetadataModel(this);
-    m_artistsModel = new PmcMetadataModel(this);
-    m_musicModel = new PmcMetadataModel(this);
+    m_albumsModel = new AlwaysExpandedMetadataModel(this);
+    m_artistsModel = new AlwaysExpandedMetadataModel(this);
+    m_musicModel = new MetadataBackendCommonModel(this);
     m_artistsModel->setDefaultDecoration("user-identity");
-    m_albumsModel->setDefaultDecoration("media-optical-audio");
-    m_albumsModel->showMediaForProperty(Nepomuk2::Vocabulary::NMM::musicAlbum());
-    m_artistsModel->showMediaForProperty(Nepomuk2::Vocabulary::NMM::performer());
-    m_musicModel->showMediaType(MediaCenter::Music);
-    emit musicModelChanged();
+    m_albumsModel->setDefaultDecoration("nocover");
     connect(m_musicModel, SIGNAL(modelReset()), SLOT(musicModelReset()));
 
-    updateModelAccordingToFilters();
+    m_musicModelMetadata = new ModelMetadata(m_musicModel, this);
+    m_albumsModelMetadata = new ModelMetadata(m_albumsModel, this);
+    m_artistsModelMetadata = new ModelMetadata(m_artistsModel, this);
+
+    m_albumsModelMetadata->setName("Albums");
+    m_artistsModelMetadata->setName("Artists");
+    m_musicModelMetadata->setName("Songs#list");
+
+    m_musicModelMetadata->setSupportsSearch(true);
+    m_artistsModelMetadata->setSupportsSearch(true);
+    m_albumsModelMetadata->setSupportsSearch(true);
+
+    addModel(m_musicModelMetadata);
+    addModel(m_albumsModelMetadata);
+    addModel(m_artistsModelMetadata);
+
+    QTimer::singleShot(1000, this, SLOT(initializeModels()));
     return true;
 }
 
-QString MetadataMusicBackend::mediaBrowserOverride() const
+void MetadataMusicBackend::initializeModels()
 {
-    return constructQmlSource("metadatamusiccomponents", "0.1", "MediaBrowser");
+    m_albumsModel->showMediaForProperty(Nepomuk2::Vocabulary::NMM::musicAlbum());
+    m_artistsModel->showMediaForProperty(Nepomuk2::Vocabulary::NMM::performer());
+    m_musicModel->showMediaType(MediaCenter::Music);
+    updateModelAccordingToFilters();
 }
 
 bool MetadataMusicBackend::supportsSearch() const
@@ -76,16 +104,6 @@ bool MetadataMusicBackend::supportsSearch() const
     return false;
 }
 
-QObject* MetadataMusicBackend::albumsModel() const
-{
-    return m_albumsModel;
-}
-
-QObject* MetadataMusicBackend::artistsModel() const
-{
-    return m_artistsModel;
-}
-
 QString MetadataMusicBackend::albumFilter() const
 {
     return m_albumFilter;
@@ -114,23 +132,28 @@ void MetadataMusicBackend::setArtistFilter(const QString& filter)
     updateModelAccordingToFilters();
 }
 
+void MetadataMusicBackend::clearFilters()
+{
+    m_musicModelMetadata->setHeaderText(QString());
+    m_musicModel->clearAllFilters();
+}
+
 void MetadataMusicBackend::updateModelAccordingToFilters()
 {
-    m_musicModel->clearAllFilters();
+    clearFilters();
 
     if (!m_albumFilter.isEmpty()) {
+        Nepomuk2::Resource r(m_albumFilter);
+        m_musicModelMetadata->setHeaderText(QString("Showing music from %1, click %2 to reset").arg(r.genericLabel()).arg(s_showAllButton));
         m_musicModel->addFilter(Nepomuk2::Vocabulary::NMM::musicAlbum(), Nepomuk2::Query::ResourceTerm(m_albumFilter));
     }
     if (!m_artistFilter.isEmpty()) {
+        Nepomuk2::Resource r(m_artistFilter);
+        m_musicModelMetadata->setHeaderText(QString("Showing music from %1, click %2 to reset").arg(r.genericLabel()).arg(s_showAllButton));
         m_musicModel->addFilter(Nepomuk2::Vocabulary::NMM::performer(), Nepomuk2::Query::ResourceTerm(m_artistFilter));
     }
 }
 
-QObject* MetadataMusicBackend::musicModel() const
-{
-    return m_musicModel;
-}
-
 void MetadataMusicBackend::searchAlbum(const QString& album)
 {
     m_albumsModel->setSearchTerm(album);
@@ -146,7 +169,7 @@ void MetadataMusicBackend::searchMusic(const QString& music)
     m_musicModel->setSearchTerm(music);
 }
 
-void MetadataMusicBackend::addAllSongsToPlaylist ( QObject* playlistModel )
+void MetadataMusicBackend::addAllSongsToPlaylist (QObject* playlistModel)
 {
     m_shallAddMediaToPlaylist = true;
     m_playlistModel = qobject_cast<PlaylistModel*>(playlistModel);
@@ -187,6 +210,59 @@ void MetadataMusicBackend::stopAddingSongsToPlaylist()
     m_shallAddMediaToPlaylist = false;
 }
 
+bool MetadataMusicBackend::expand(int row, QAbstractItemModel* model)
+{
+    if (!model) return false;
 
+    const QString filter = model->data(model->index(row, 0), MediaCenter::ResourceIdRole).toString();
+
+    if (model == m_albumsModel) {
+        setAlbumFilter(filter);
+        emit modelNeedsAttention(m_musicModelMetadata);
+    } else if (model == m_artistsModel) {
+        setArtistFilter(filter);
+        emit modelNeedsAttention(m_musicModelMetadata);
+    }
+
+    return true;
+}
+
+QVariantList MetadataMusicBackend::buttons()
+{
+    QVariantList buttonList;
+    buttonList << s_showAllButton << s_playAllButton;
+    return buttonList;
+}
+
+void MetadataMusicBackend::handleButtonClick(const QString& buttonName)
+{
+    if (buttonName == s_showAllButton) {
+        m_albumFilter = m_artistFilter = "";
+        updateModelAccordingToFilters();
+    } else if (buttonName == s_playAllButton) {
+        //FIXME: This is a horrible hack to get a ref to the playlist model
+        PlaylistModel *model = qobject_cast<PlaylistModel*>(declarativeEngine()->rootContext()->contextProperty("playlistModel").value<QObject*>());
+        if (model) {
+            addAllSongsToPlaylist(model);
+            if (pmcRuntime()) {
+                QObject *playlist = pmcRuntime()->property("playlist").value<QObject*>();
+                QMetaObject::invokeMethod(playlist, "playNext");
+            }
+        } else {
+            kWarning() << "Failed to get a reference to playlist model";
+        }
+    }
+}
+
+void MetadataMusicBackend::searchModel(const QString& searchTerm, QAbstractItemModel* model)
+{
+    if (model == m_musicModel) {
+        searchMusic(searchTerm);
+    } else if (model == m_artistsModel) {
+        searchArtist(searchTerm);
+    } else if (model == m_albumsModel) {
+        searchAlbum(searchTerm);
+    }
+}
 
 #include "metadatamusicbackend.moc"
