@@ -21,7 +21,6 @@
 
 
 #include "metadatavideobackend.h"
-
 #include "metadatavideomodel.h"
 
 #include <Nepomuk2/Query/QueryParser>
@@ -42,9 +41,11 @@ MetadataVideoBackend::~MetadataVideoBackend()
 bool MetadataVideoBackend::initImpl()
 {
     AbstractMetadataBackend::initImpl();
-    PmcMetadataModel *pmcMetadataModel = new MetadataVideoModel(this);
-    handleBusySignals(pmcMetadataModel);
-    setModel(pmcMetadataModel);
+    MetadataVideoModel *model = new MetadataVideoModel(this);
+    ModelMetadata *metadata = new ModelMetadata(model, this);
+    metadata->setSupportsSearch(true);
+    handleBusySignals(model);
+    setModel(metadata);
     return true;
 }
 
