@@ -17,6 +17,8 @@
  ***********************************************************************************/
 
 #include "abstractbrowsingbackend.h"
+#include "objectpair.h"
+#include "modelmetadata.h"
 
 #include <KGlobal>
 #include <KService>
@@ -45,8 +47,9 @@ public:
     bool hasInitialized;
     QString name;
     QString mediaBrowserSidePanelText;
-    QList<QAbstractItemModel*> models;
+    QList<QObject*> models;
     QString searchTerm;
+    QObject* pmcRuntime;
 };
 
 AbstractBrowsingBackend::AbstractBrowsingBackend(QObject *parent, const QVariantList &args)
@@ -77,17 +80,41 @@ QString AbstractBrowsingBackend::name() const
     return d->name;
 }
 
-void AbstractBrowsingBackend::setModel(QAbstractItemModel * model)
+void AbstractBrowsingBackend::setModel(ModelMetadata* model)
 {
     d->models.clear();
-    d->models.append(model);
-    emit modelsChanged();
-    emit modelChanged();
+    addModel(model);
 }
 
-QObject * AbstractBrowsingBackend::model()
+void AbstractBrowsingBackend::setModel(QAbstractItemModel* model)
 {
-    return d->models.length() ? (QObject*)(d->models.first()) : 0;
+    ModelMetadata *metadata = new ModelMetadata(model, this);
+    setModel(metadata);
+}
+
+void AbstractBrowsingBackend::addModel(ModelMetadata* model)
+{
+    d->models.append(model);
+    emit modelsChanged();
+}
+
+void AbstractBrowsingBackend::addModelPair(const QString& pairLabel, QObject* firstModel, QObject* secondModel)
+{
+    ObjectPair *modelPair = new ObjectPair(this);
+    modelPair->setFirst(firstModel);
+    modelPair->setSecond(secondModel);
+
+    modelPair->setObjectName(pairLabel);
+    d->models.append(modelPair);
+}
+
+QAbstractItemModel* AbstractBrowsingBackend::model()
+{
+    QObject *model = d->models.length() ? (QObject*)(d->models.first()) : 0;
+    if (model) {
+        return qobject_cast<QAbstractItemModel*>(qobject_cast<ModelMetadata*>(model)->model());
+    }
+    return 0;
 }
 
 KService::List AbstractBrowsingBackend::availableBackends()
@@ -104,9 +131,13 @@ bool AbstractBrowsingBackend::goOneLevelUp()
     return false;
 }
 
+bool AbstractBrowsingBackend::expand(int row, QAbstractItemModel* model)
+{
+    return expand(row);
+}
+
 bool AbstractBrowsingBackend::expand(int row)
 {
-    Q_UNUSED(row)
     return false;
 }
 
@@ -160,6 +191,12 @@ void AbstractBrowsingBackend::search(const QString& searchTerm)
     // Does nothing
 }
 
+void AbstractBrowsingBackend::searchModel(const QString& searchTerm, QAbstractItemModel* model)
+{
+    Q_UNUSED(model)
+    search(searchTerm);
+}
+
 bool AbstractBrowsingBackend::init()
 {
     if (!d->hasInitialized) {
@@ -176,7 +213,7 @@ bool AbstractBrowsingBackend::busy() const
 QVariantList AbstractBrowsingBackend::models()
 {
     QVariantList modelList;
-    Q_FOREACH(QAbstractItemModel *model, d->models) {
+    Q_FOREACH(QObject *model, d->models) {
         modelList.append(QVariant::fromValue(qobject_cast<QObject*>(model)));
     }
     return modelList;
@@ -195,3 +232,25 @@ void AbstractBrowsingBackend::setSearchTerm(const QString& term)
     emit searchTermChanged();
     search(term);
 }
+
+QVariantList AbstractBrowsingBackend::buttons()
+{
+    return QVariantList();
+}
+
+void AbstractBrowsingBackend::handleButtonClick(const QString& buttonName)
+{
+}
+
+QObject* AbstractBrowsingBackend::pmcRuntime()
+{
+    return d->pmcRuntime;
+}
+
+void AbstractBrowsingBackend::setPmcRuntime(QObject* pmcRuntime)
+{
+    if (d->pmcRuntime != pmcRuntime) {
+        d->pmcRuntime = pmcRuntime;
+        emit pmcRuntimeChanged();
+    }
+}
