@@ -24,13 +24,15 @@
 #include <QFlags>
 
 #include <KDE/KConfigGroup>
+#include <KDE/KService>
+
 #include "mediacenter_export.h"
 #include "mediacenter/mediacenter.h"
-#include <KDE/KService>
 
 class QDeclarativeEngine;
 class QAbstractItemModel;
 class KConfigDialog;
+class ModelMetadata;
 
 namespace MediaCenter {
 
@@ -42,18 +44,19 @@ namespace MediaCenter {
  * Custom models are useful to implement custom navigations such as navigation
  * through services available through the web, see YouTube.
  *
- * This class does not direclty inherit from QAbstractItemModel due to future
+ * This class does not directly inherit from QAbstractItemModel due to future
  * extension purposes. It just represents an interface between the applet and the
  * provided model.
  */
 class MEDIACENTER_EXPORT AbstractBrowsingBackend : public QObject
 {
     Q_OBJECT
-    Q_PROPERTY(QObject* backendModel READ model NOTIFY modelChanged)
     Q_PROPERTY(bool busy READ busy NOTIFY busyChanged)
     Q_PROPERTY(QString mediaBrowserSidePanel READ mediaBrowserSidePanel WRITE setMediaBrowserSidePanel NOTIFY mediaBrowserSidePanelChanged)
     Q_PROPERTY(QVariantList models READ models NOTIFY modelsChanged)
     Q_PROPERTY(QString searchTerm READ searchTerm WRITE setSearchTerm NOTIFY searchTermChanged)
+    Q_PROPERTY(QVariantList buttons READ buttons NOTIFY buttonsChanged)
+    Q_PROPERTY(QObject* pmcRuntime READ pmcRuntime WRITE setPmcRuntime NOTIFY pmcRuntimeChanged)
 
 public:
     /**
@@ -75,15 +78,21 @@ public:
     QString name() const;
 
     /**
-     * @returns the model of the browsing backend
+     * @returns the model of the browsing backend.
+     * The default implementation returns the first (or the only) model.
      */
-    QObject *model();
+    QAbstractItemModel *model();
 
     /**
      * @returns the models available in the browsing backend
      */
     QVariantList models();
 
+    /**
+     * @returns a list of buttons the model wants to show on the UI
+     */
+    virtual QVariantList buttons();
+
     /**
      * Request the backend to initialize
      */
@@ -110,6 +119,18 @@ public:
      * to a particular one
      *
      * @param row the row index of the directory
+     * @param model the current model that the backend should operate on
+     *
+     * @return true if operation was successful
+     * @return false if operation was unsuccessful
+     */
+    Q_INVOKABLE virtual bool expand(int row, QAbstractItemModel *model);
+
+    /**
+     * This is a convenience method for when the media browser thinks the model
+     * doesn't matter. In this case, the default model is assumed by the backend.
+     *
+     * @param row the row index of the directory
      *
      * @return true if operation was successful
      * @return false if operation was unsuccessful
@@ -173,27 +194,56 @@ public:
      */
     virtual bool busy() const;
 
+    /**
+     * This method will be called by mediacenter when one of the buttons is clicked.
+     * The default implementation does nothing
+     *
+     * @param buttonName the name of the button that was clicked
+     */
+    Q_INVOKABLE virtual void handleButtonClick(const QString &buttonName);
+
     QString searchTerm() const;
     void setSearchTerm(const QString &term);
 
+    QObject* pmcRuntime();
+    void setPmcRuntime(QObject* pmcRuntime);
+
 Q_SIGNALS:
-    void modelChanged();
     void busyChanged();
     void mediaBrowserSidePanelChanged();
     void modelsChanged();
+    void buttonsChanged();
     void searchTermChanged();
+    void error(const QString &message);
+    void modelNeedsAttention(QObject* model);
+    void pmcRuntimeChanged();
 
 protected:
     /**
      * This method must be set in order to provide the model to be
      * used by the view.
-     * @note When LocalBrowsing is set it is highly recommended to use a KDirModel.     ////TODO: May no longer be needed
-     * If the package is set to RemoteBrowsing the model
-     * must provide an url that points to the media content for each
-     * QModelIndex. It must make use of MediaRole role to accomplish this.
      * @see MediaRole
      */
-    void setModel(QAbstractItemModel * model);
+    void setModel(ModelMetadata* model);
+
+    /**
+     * This method must be set in order to provide the model to be
+     * used by the view.
+     * @see MediaRole
+     */
+    void setModel(QAbstractItemModel* model);
+
+    /**
+     * This method is used to add more models to this backend
+     * @see MediaRole
+     */
+    void addModel(ModelMetadata * model);
+
+    /**
+     * This method adds multiple models as a single model collection.
+     * This indicates the UI to show views for these models simultaneously
+     */
+    void addModelPair(const QString& pairLabel, QObject* firstModel, QObject* secondModel);
 
     /**
      * This is a convenience function which constructs a string representing QML source for
@@ -219,8 +269,16 @@ protected:
      *
      * @param searchTerm string entered by the user
      */
-    virtual void search(const QString &searchTerm);
+    Q_INVOKABLE virtual void search(const QString &searchTerm);
 
+    /**
+     * This method is called by the UI when the user requests to search for a media
+     * for a particular model
+     *
+     * @param searchTerm string entered by the user
+     * @param model the model that this search should affect
+     */
+    Q_INVOKABLE virtual void searchModel(const QString &searchTerm, QAbstractItemModel *model);
 
 private:
     class AbstractBrowsingBackendPrivate;
@@ -230,6 +288,8 @@ private:
 
 } // MediaCenter namespace
 
+Q_DECLARE_METATYPE(QAbstractItemModel*)
+
 #define MEDIACENTER_EXPORT_BROWSINGBACKEND( c ) \
         K_PLUGIN_FACTORY( MediaBrowserFactory, registerPlugin< c >(); ) \
         K_EXPORT_PLUGIN( MediaBrowserFactory("c") )
